////////////////////////////////////////////////////////////////////////////////
// Project:	DS1922						      
// Module:      cmd_ctrl                                                
// Path:	/tds/cmd_ctrl/src/cmd_ctrl.vb			    
// Designer:	Michael Haight
// Date:        05/29/01        
//
// Description: This is the behavioral model for the command control center of 
//		the DS1922 data logger chip.
//////////////////////////////////////////////////////////////////////////////// 
module cmd_ctrl (
  CLK_MEM, IOX_RSTZ, IOX_WRDATA, END_1WIRE,
           IOX_CSR,  IOX_RDDATA, IOX_READZ);


///// Port Declarations ////////////////////////////////////////////////////////

input		CLK_MEM;
input		IOX_RSTZ;
input		IOX_WRDATA;

output		END_1WIRE;
output		IOX_CSR;
output		IOX_RDDATA;
output		IOX_READZ;


//-------------------------------
// JJG Debug
wire IOX_CSR = 1'b0;
wire WAKE = 1'b1;
//-------------------------------





///// General Declarations /////////////////////////////////////////////////////
event	start_cmd;

integer	                bit;
integer byte;
integer	i;
integer first_pass;

parameter [7:0] WRITE_SP     	= 8'h0F,
		READ_SP      	= 8'hAA,
		COPY_SP      	= 8'h55,
		READ_MEM	= 8'hF0;


reg write_sp_flag;
reg read_sp_flag;
reg copy_sp_flag;

reg		AA;
reg	[1:0]	ADDR_DEC;
reg	[7:0]	ALARM_STATUS;
reg		BEGIN_CIP;
reg		BIST_CONT;
reg		BIST_RUN;
reg		BOR_FLAG;
reg		CIP;
reg		CLR_ALARMS;
reg		CLR_MEMZ;
reg		CLR_ON_SM;
reg	[13:0]	COMM_ADDR;
reg		COMM_MEZ;
reg		COMM_OEZ;
reg		COMM_WEZ;
reg	[7:0]	COMM_WRDATA;
reg	[13:0]	DATA_ADDR;
reg	[23:0]	DEV_CNT;
reg	[7:0]	DEVICE_CONFIG;
reg	[7:0]	DEVICE_SAMPLE_L;
reg	[7:0]	DEVICE_SAMPLE_M;
reg	[7:0]	DEVICE_SAMPLE_H;
reg		EN_DATA_ADDR;
reg		EN_RING;
reg		EN_TEMP_ADDR;
reg		END_1WIRE;
reg	[7:0]	ES;
reg		FCIP;
reg		FORCE_CONV;
reg	[7:0]	GENERAL_STATUS;
reg	[7:0]	GENERAL_TEST_W;
reg		INC_DEVICE;
reg		INC_MISSION;
reg		IOX_RDDATA;
reg		IOX_READ;
reg		LD_DATA_ADDR;
reg		LD_TEMP_ADDR;
reg		MAXD_ADDR;
reg		MAXT_ADDR;
reg		MEMCLR;
reg		MEM_LOCK;
reg		MIP;
reg		MIP_WFTA;
reg	[13:0]	MISSION_ADDR;
reg	[7:0]	MISSION_CONTROL;
reg		MISSION_MEZ;
reg		MISSION_OEZ;
reg	[7:0]	MISSION_SAMPLE_L;
reg	[7:0]	MISSION_SAMPLE_M;
reg	[7:0]	MISSION_SAMPLE_H;
reg		MISSION_WEZ;
reg	[7:0]	MISSION_WRDATA;
reg	[23:0]	MSN_CNT;
reg	[5:0]	OFFSET;
reg		OFST_FLAG;
reg	[7:0]	OPCODE;
reg	[7:0]	PASSWORDA_0;
reg	[7:0]	PASSWORDA_1;
reg	[7:0]	PASSWORDA_2;
reg	[7:0]	PASSWORDA_3;
reg	[7:0]	PASSWORDA_4;
reg	[7:0]	PASSWORDA_5;
reg	[7:0]	PASSWORDA_6;
reg	[7:0]	PASSWORDA_7;
reg	[7:0]	PASSWORDB_0;
reg	[7:0]	PASSWORDB_1;
reg	[7:0]	PASSWORDB_2;
reg	[7:0]	PASSWORDB_3;
reg	[7:0]	PASSWORDB_4;
reg	[7:0]	PASSWORDB_5;
reg	[7:0]	PASSWORDB_6;
reg	[7:0]	PASSWORDB_7;
reg	[7:0]	PASSWORD_CONTROL;
reg		PF;
reg	[7:0]	READ_PASSWORD;
reg		SAMPLE;
reg		SEL_MEM;
reg		SIP;
reg		SLEEP;
reg	[7:0]	SFR_RDDATA;
reg	[4:0]	SP_ADDR;
reg		SP_MEZ;
reg		SP_OEZ;
reg		SP_WEZ;
reg	[7:0]	SP_WRDATA;
reg		START_DCONV;
reg		START_MISSION;
reg		START_TCONV;
reg	[7:0]	STATUS1;
reg	[7:0]	STATUS2;
reg		STOP_MISSION;
reg	[15:0]	TA;
reg	[13:0]	TEMP_ADDR;
reg		WFTA;
reg	[7:0]	WORD;

wire		AD_VALID;
wire		ALARM_RES;
wire	[13:0]	CMD_ADDR;
wire		CMD_MEZ;
wire		CMD_OEZ;
wire		CMD_WEZ;
wire	[7:0]	CMD_WRDATA;
wire		DHF;
wire		DLBS;
wire		DLF;
wire		EDL;
wire		EN_PASSWORDS;
wire		EPCK;
wire		ETL;
wire	[7:0]	GENERAL_TEST;
//wire		IOX_CSR;
wire		IOX_READZ;
wire	[63:0]	PASSWORDA;
wire	[63:0]	PASSWORDB;
wire		RD_EN;
wire	[7:0]	RDDATA;
wire		RO;
wire		SFR_SPACE;
wire		SUTA;
wire		TEST;
wire		THF;
wire		TLBS;
wire		TLF;

reg		maxd_flag;
reg		maxt_flag;

reg		READMEM;

////////////////////////////////////////////////////////////////////////////////
assign	IOX_READZ = ~IOX_READ;

assign	EN_PASSWORDS = (PASSWORD_CONTROL == 8'hAA);
assign	ETL  = MISSION_CONTROL[0];
assign	EDL  = MISSION_CONTROL[1];
assign	TLBS = MISSION_CONTROL[2];
assign	DLBS = MISSION_CONTROL[3];
assign	RO   = MISSION_CONTROL[4];
assign  SFR_SPACE = (TA >= 14'h0200 && TA <= 14'h023F);
assign	SUTA = MISSION_CONTROL[5];
assign	GENERAL_TEST[7:5] = 3'b111;
assign	GENERAL_TEST[4] = (TEST && ~MEM_LOCK && GENERAL_TEST_W[4]);
assign	GENERAL_TEST[3:0] = MEM_LOCK ? 4'hA : GENERAL_TEST_W[3:0];
assign	EPCK = GENERAL_TEST[4];

assign CMD_ADDR = END_1WIRE ? MISSION_ADDR : COMM_ADDR;
assign CMD_MEZ  = END_1WIRE ? MISSION_MEZ  : COMM_MEZ;
assign CMD_OEZ  = END_1WIRE ? MISSION_OEZ  : COMM_OEZ;
assign CMD_WEZ  = END_1WIRE ? MISSION_WEZ  : COMM_WEZ;
assign CMD_WRDATA  = END_1WIRE ? MISSION_WRDATA  : COMM_WRDATA;

//-------------------------------------------
//ADDED JJG

reg BOR;
initial begin 

   BOR = 1'b1;
   
end    
//-------------------------------------------

always @(GENERAL_TEST)
  if (GENERAL_TEST[3:0] == 4'hA)
    MEM_LOCK = 1;
  else
    MEM_LOCK = 0;

///// combinational model for status bits //////////////////////////////////////

always @(posedge BOR or negedge CLR_MEMZ or posedge BOR_FLAG)
begin
  if (BOR)
  begin
    BOR_FLAG <= 1;
  end
  if (~CLR_MEMZ)
  begin
    BOR_FLAG <= 0;
  end
end

assign TLF = ALARM_STATUS[0];
assign THF = ALARM_STATUS[1];
assign DLF = ALARM_STATUS[2];
assign DHF = ALARM_STATUS[3];

initial
begin
  AA		= 0;
  BEGIN_CIP	= 0;
  BIST_RUN	= 0;
  BIST_CONT	= 0;
  CIP		= 0;
  CLR_MEMZ	= 1;
  COMM_ADDR	= 14'h0;
  COMM_MEZ	= 1;
  COMM_OEZ	= 1;
  COMM_WEZ	= 1;
  COMM_WRDATA	= 8'h0;
  EN_RING	= 0;
  END_1WIRE	= 1;
  ES		= 0;
  FCIP		= 0;
  FORCE_CONV	= 0;
  IOX_RDDATA	= 0;
  IOX_READ	= 0;
  MEMCLR	= 0;
  MEM_LOCK	= 0;
  MIP		= 0;
  MIP_WFTA	= 0;
  MISSION_ADDR  = 14'h0;
  MISSION_MEZ	= 1;
  MISSION_OEZ	= 1;
  MISSION_WEZ	= 1;
  MISSION_WRDATA= 8'h0;
  OFFSET	= 5'h0;
  OFST_FLAG 	= 0;
  OPCODE	= 8'h0;
  PF		= 0;
  READ_PASSWORD	= 8'h0;
  SAMPLE	= 0;
  SEL_MEM	= 0;
  SIP		= 0;
  SP_ADDR	= 5'b0;
  SP_MEZ	= 1;
  SP_OEZ	= 1;
  SP_WEZ	= 1;
  SP_WRDATA	= 8'h0;
  START_MISSION = 0;
  STOP_MISSION  = 1;
  TA		= 16'h0;
  WORD		= 8'h0;
  WFTA		= 0;

  first_pass	= 1;
  maxd_flag	= 0;
  maxt_flag	= 0;
  READMEM	= 0;
end

//Removed JJG

/*
always @(posedge BOR)		// Initialize SFRs
begin
  if (BOR)
  begin
    ALARM_STATUS	= 8'hF0;
    DEV_CNT		= 24'h000000;
    DEVICE_CONFIG	= 8'h0;
    DEVICE_SAMPLE_L	= 8'h0;
    DEVICE_SAMPLE_M	= 8'h0;
    MISSION_SAMPLE_H	= 8'h0;
    GENERAL_STATUS	= 8'hC0;
    GENERAL_TEST_W	= 8'h00;
    MISSION_CONTROL	= 8'hC0;
    DEVICE_SAMPLE_L	= 8'h0;
    DEVICE_SAMPLE_M	= 8'h0;
    MISSION_SAMPLE_H	= 8'h0;
    MSN_CNT		= 24'h000000;
    PASSWORDA_0		= 8'h0;
    PASSWORDA_1		= 8'h0;
    PASSWORDA_2		= 8'h0;
    PASSWORDA_3		= 8'h0;
    PASSWORDA_4		= 8'h0;
    PASSWORDA_5		= 8'h0;
    PASSWORDA_6		= 8'h0;
    PASSWORDA_7		= 8'h0;
    PASSWORDB_0		= 8'h0;
    PASSWORDB_1		= 8'h0;
    PASSWORDB_2		= 8'h0;
    PASSWORDB_3		= 8'h0;
    PASSWORDB_4		= 8'h0;
    PASSWORDB_5		= 8'h0;
    PASSWORDB_6		= 8'h0;
    PASSWORDB_7		= 8'h0;
    PASSWORD_CONTROL	= 8'h0;
  end
end
*/


//Removed JJG
/*
assign ALARM_RES = (ALARM_STATUS[0] || ALARM_STATUS[1] || ALARM_STATUS[2] ||
		    ALARM_STATUS[3] || ALARM_STATUS[7]);

assign IOX_CSR = ALARM_RES;
*/

//Removed JJG
/*
always @(negedge CLK_MEM)
begin
  ALARM_STATUS[0]	<= (((TEMP_LO_ALARM && (MIP || FCIP) && ~BEGIN_CIP) || 
			      ALARM_STATUS[0]) && CLR_MEMZ);
  ALARM_STATUS[1]	<= (((TEMP_HI_ALARM && (MIP || FCIP) && ~BEGIN_CIP) ||
 			      ALARM_STATUS[1]) && CLR_MEMZ);
  ALARM_STATUS[2]	<= (((DATA_LO_ALARM && (MIP || FCIP) && ~BEGIN_CIP) ||
 			      ALARM_STATUS[2]) && CLR_MEMZ);
  ALARM_STATUS[3]	<= (((DATA_HI_ALARM && (MIP || FCIP) && ~BEGIN_CIP) ||
 			      ALARM_STATUS[3]) && CLR_MEMZ);
  ALARM_STATUS[6:4]	<= 3'b111;
  ALARM_STATUS[7]	<= BOR_FLAG;
  GENERAL_STATUS[0]	<= 0;
  GENERAL_STATUS[1]	<= MIP;
  GENERAL_STATUS[2]	<= 0;
  GENERAL_STATUS[3]	<= MEMCLR;
  GENERAL_STATUS[4]	<= WFTA;
  GENERAL_STATUS[5]	<= FCIP;
  GENERAL_STATUS[7:6]	<= 2'b11;

  if (~CLR_MEMZ || CLR_ON_SM)
  begin
    DEVICE_SAMPLE_L = 0;
    DEVICE_SAMPLE_M = 0;
    MISSION_SAMPLE_H = 0;
  end
  if (TEST & ~COMM_MEZ & ~COMM_WEZ)		// Write access in test
    case (COMM_ADDR)
      GENERAL_TEST_SFR	   : GENERAL_TEST_W   = CMD_WRDATA;
      DEVICE_CONFIG_SFR	   : if (TEST) DEVICE_CONFIG = CMD_WRDATA;
      DEVICE_SAMPLE_L_SFR  : DEVICE_SAMPLE_L  = CMD_WRDATA;
      DEVICE_SAMPLE_M_SFR  : DEVICE_SAMPLE_M  = CMD_WRDATA;
      DEVICE_SAMPLE_H_SFR  : DEVICE_SAMPLE_H  = CMD_WRDATA;
      MISSION_CONTROL_SFR  : MISSION_CONTROL  = {2'h3,CMD_WRDATA[5:0]};
      MISSION_SAMPLE_L_SFR : MISSION_SAMPLE_L = CMD_WRDATA;
      MISSION_SAMPLE_M_SFR : MISSION_SAMPLE_M = CMD_WRDATA;
      MISSION_SAMPLE_H_SFR : MISSION_SAMPLE_H = CMD_WRDATA;
      PASSWORDA_0_SFR	   : PASSWORDA_0      = CMD_WRDATA;
      PASSWORDA_1_SFR	   : PASSWORDA_1      = CMD_WRDATA;
      PASSWORDA_2_SFR	   : PASSWORDA_2      = CMD_WRDATA;
      PASSWORDA_3_SFR	   : PASSWORDA_3      = CMD_WRDATA;
      PASSWORDA_4_SFR	   : PASSWORDA_4      = CMD_WRDATA;
      PASSWORDA_5_SFR	   : PASSWORDA_5      = CMD_WRDATA;
      PASSWORDA_6_SFR	   : PASSWORDA_6      = CMD_WRDATA;
      PASSWORDA_7_SFR	   : PASSWORDA_7      = CMD_WRDATA;
      PASSWORDB_0_SFR	   : PASSWORDB_0      = CMD_WRDATA;
      PASSWORDB_1_SFR	   : PASSWORDB_1      = CMD_WRDATA;
      PASSWORDB_2_SFR	   : PASSWORDB_2      = CMD_WRDATA;
      PASSWORDB_3_SFR	   : PASSWORDB_3      = CMD_WRDATA;
      PASSWORDB_4_SFR	   : PASSWORDB_4      = CMD_WRDATA;
      PASSWORDB_5_SFR	   : PASSWORDB_5      = CMD_WRDATA;
      PASSWORDB_6_SFR	   : PASSWORDB_6      = CMD_WRDATA;
      PASSWORDB_7_SFR	   : PASSWORDB_7      = CMD_WRDATA;
      PASSWORD_CTRL_SFR    : PASSWORD_CONTROL = CMD_WRDATA;
    endcase
  else if (~COMM_MEZ & ~COMM_WEZ && ~TEST)	// Always full Write access
    case (COMM_ADDR)
      GENERAL_TEST_SFR	   : GENERAL_TEST_W   = CMD_WRDATA;
      MISSION_CONTROL_SFR  : MISSION_CONTROL  = {2'h3,CMD_WRDATA[5:0]};
      PASSWORDA_0_SFR	   : PASSWORDA_0      = CMD_WRDATA;
      PASSWORDA_1_SFR	   : PASSWORDA_1      = CMD_WRDATA;
      PASSWORDA_2_SFR	   : PASSWORDA_2      = CMD_WRDATA;
      PASSWORDA_3_SFR	   : PASSWORDA_3      = CMD_WRDATA;
      PASSWORDA_4_SFR	   : PASSWORDA_4      = CMD_WRDATA;
      PASSWORDA_5_SFR	   : PASSWORDA_5      = CMD_WRDATA;
      PASSWORDA_6_SFR	   : PASSWORDA_6      = CMD_WRDATA;
      PASSWORDA_7_SFR	   : PASSWORDA_7      = CMD_WRDATA;
      PASSWORDB_0_SFR	   : PASSWORDB_0      = CMD_WRDATA;
      PASSWORDB_1_SFR	   : PASSWORDB_1      = CMD_WRDATA;
      PASSWORDB_2_SFR	   : PASSWORDB_2      = CMD_WRDATA;
      PASSWORDB_3_SFR	   : PASSWORDB_3      = CMD_WRDATA;
      PASSWORDB_4_SFR	   : PASSWORDB_4      = CMD_WRDATA;
      PASSWORDB_5_SFR	   : PASSWORDB_5      = CMD_WRDATA;
      PASSWORDB_6_SFR	   : PASSWORDB_6      = CMD_WRDATA;
      PASSWORDB_7_SFR	   : PASSWORDB_7      = CMD_WRDATA;
      PASSWORD_CTRL_SFR    : PASSWORD_CONTROL = CMD_WRDATA;
    endcase
  else if (~COMM_MEZ & ~COMM_OEZ)		// Always full read access
    case (COMM_ADDR)
      ALARM_STATUS_SFR	   : SFR_RDDATA = ALARM_STATUS;
      DEVICE_CONFIG_SFR	   : SFR_RDDATA = DEVICE_CONFIG;
      DEVICE_SAMPLE_L_SFR  : SFR_RDDATA = DEVICE_SAMPLE_L;
      DEVICE_SAMPLE_M_SFR  : SFR_RDDATA = DEVICE_SAMPLE_M;
      DEVICE_SAMPLE_H_SFR  : SFR_RDDATA = DEVICE_SAMPLE_H;
      GENERAL_STATUS_SFR   : SFR_RDDATA = GENERAL_STATUS;
      GENERAL_TEST_SFR	   : SFR_RDDATA = {3'b111, GENERAL_TEST[4:0]};
      MISSION_CONTROL_SFR  : SFR_RDDATA = MISSION_CONTROL;
      MISSION_SAMPLE_L_SFR : SFR_RDDATA = MISSION_SAMPLE_L;
      MISSION_SAMPLE_M_SFR : SFR_RDDATA = MISSION_SAMPLE_M;
      MISSION_SAMPLE_H_SFR : SFR_RDDATA = MISSION_SAMPLE_H;
      PASSWORD_CTRL_SFR    : SFR_RDDATA = (~READ_PASSWORD | PASSWORD_CONTROL);
      default		   : SFR_RDDATA = 8'h00;
    endcase
  else
  begin
    DEVICE_SAMPLE_L  = DEV_CNT[7:0];
    DEVICE_SAMPLE_M  = DEV_CNT[15:8];
    DEVICE_SAMPLE_H  = DEV_CNT[23:16];
    MISSION_SAMPLE_L  = MSN_CNT[7:0];
    MISSION_SAMPLE_M  = MSN_CNT[15:8];
    MISSION_SAMPLE_H  = MSN_CNT[23:16];
  end    
end

*/

//Removed jjg
/*
assign AD_VALID = ((COMM_ADDR == ALARM_STATUS_SFR)	|
		   (COMM_ADDR == DEVICE_CONFIG_SFR)	|
                   (COMM_ADDR == DEVICE_SAMPLE_L_SFR)	|
                   (COMM_ADDR == DEVICE_SAMPLE_M_SFR)	|
                   (COMM_ADDR == DEVICE_SAMPLE_H_SFR)	|
                   (COMM_ADDR == GENERAL_STATUS_SFR)	|
                   (COMM_ADDR == GENERAL_TEST_SFR)	|
                   (COMM_ADDR == MISSION_CONTROL_SFR)	|
                   (COMM_ADDR == MISSION_SAMPLE_L_SFR)	|
                   (COMM_ADDR == MISSION_SAMPLE_M_SFR)	|
                   (COMM_ADDR == MISSION_SAMPLE_H_SFR)	|
                   (COMM_ADDR == PASSWORDA_0_SFR)	|
                   (COMM_ADDR == PASSWORDA_1_SFR)	|
                   (COMM_ADDR == PASSWORDA_2_SFR)	|
                   (COMM_ADDR == PASSWORDA_3_SFR)	|
                   (COMM_ADDR == PASSWORDA_4_SFR)	|
                   (COMM_ADDR == PASSWORDA_5_SFR)	|
                   (COMM_ADDR == PASSWORDA_6_SFR)	|
                   (COMM_ADDR == PASSWORDA_7_SFR)	|
                   (COMM_ADDR == PASSWORDB_0_SFR)	|
                   (COMM_ADDR == PASSWORDB_1_SFR)	|
                   (COMM_ADDR == PASSWORDB_2_SFR)	|
                   (COMM_ADDR == PASSWORDB_3_SFR)	|
                   (COMM_ADDR == PASSWORDB_4_SFR)	|
                   (COMM_ADDR == PASSWORDB_5_SFR)	|
                   (COMM_ADDR == PASSWORDB_6_SFR)	|
                   (COMM_ADDR == PASSWORDB_7_SFR)	|
                   (COMM_ADDR == PASSWORD_CTRL_SFR));

*/

// Removed JJG
/*
assign RD_EN = (!COMM_MEZ & !COMM_OEZ & AD_VALID);

assign RDDATA = RD_EN ? SFR_RDDATA : CMD_RDDATA;

assign PASSWORDA = ({PASSWORDA_7, PASSWORDA_6, PASSWORDA_5, PASSWORDA_4,
		     PASSWORDA_3, PASSWORDA_2, PASSWORDA_1, PASSWORDA_0});

assign PASSWORDB = ({PASSWORDB_7, PASSWORDB_6, PASSWORDB_5, PASSWORDB_4,
		     PASSWORDB_3, PASSWORDB_2, PASSWORDB_1, PASSWORDB_0});
*/

//Removed JJG
/*
// Line up the RTC_SAMPLE pule to CLK_MEM
always @(posedge BOR or posedge CLK_MEM)
if (BOR)
  SAMPLE = 0;
else if (RTC_SAMPLE)
  SAMPLE = 1;
else
  SAMPLE = 0;
  
*/



  
/*
always @(posedge RTC_SAMPLE or posedge BOR or posedge CLK_MEM)
if (BOR)
  SAMPLE = 0;
else if (RTC_SAMPLE)
begin
  SAMPLE = 1;
  @(posedge CLK_MEM);
  @(posedge CLK_MEM);
  SAMPLE = 0;
end
*/
////////////////////////////////////////////////////////////////////////////////


always @(posedge BOR or posedge AA or posedge CLK_MEM)
  if (BOR)
    ES[7] <= 0;
  else if (AA)
    ES[7] <= 1;
  else
    ES[7] <= AA;

/*
always @(negedge IOX_RSTZ)	// Disable any current tasks running
begin
  if (WAKE)
  begin
    IOX_RDDATA = 1;
    IOX_READ = 1;
    disable start_1wire;
    END_1WIRE = 1;
    @(posedge SLEEP); 
  end
  else if (EN_RING)
  begin
    while(EN_RING)
      @(negedge CLK_MEM);
    disable start_1wire;
  end
  else
  begin
    disable start_1wire;
  end
  END_1WIRE = 1;
  #1;					// unit delay so disable and new task
  ->start_cmd;				// invocation occur in different event
					// queues.
end
*/

// -------------------------------------------------------
// --------------------  Added JJG ---------------------------
always @(negedge IOX_RSTZ)	// Disable any current tasks running
begin
    IOX_RDDATA = 1;
    IOX_READ = 1;  
    

    disable start_1wire;
    ->start_cmd;
end

/*
always @(posedge CLK_MEM)

  begin
    IOX_RDDATA = 1;
    IOX_READ = 1;
    disable start_1wire;
    ->start_cmd;
  end
*/  
// -------------------------------------------------------


//always @(negedge IOX_RSTZ)	// Disable any current tasks running

// Remove JJG
/*
always @(posedge CLK_MEM)
begin
  if (WAKE && ~EN_RING)
  begin
    IOX_RDDATA = 1;
    IOX_READ = 1;
    disable start_1wire;
    END_1WIRE = 1;
    @(posedge SLEEP); 
    ->start_cmd;
  end
  else if (WAKE && EN_RING)
  begin
    IOX_RDDATA = 1;
    IOX_READ = 1;
    @(negedge EN_RING);
    disable start_1wire;
    END_1WIRE = 1;
    @(posedge SLEEP); 
    ->start_cmd;
  end
end
*/

always @(start_cmd)			// Start processing commands
begin : start_1wire
  COMM_MEZ = 1;
  COMM_OEZ = 1;
  COMM_WEZ = 1;
  IOX_READ = 0;
  OFST_FLAG  = 0;
  SP_ADDR = 5'b0;
  SP_MEZ = 1;
  SP_OEZ = 1;
  SP_WEZ = 1;
  SP_WRDATA = 8'h0;

  @(posedge IOX_RSTZ);
  END_1WIRE = 0;

  for (bit = 0; bit <=7; bit=bit+1)
  begin
    @(posedge CLK_MEM);
    OPCODE[bit] = IOX_WRDATA;
  end

  tb_ds1wm.xtc_ds1wm.xscoreboard.verify_command(OPCODE);
  
  case (OPCODE)
    WRITE_SP:		begin
                          write_sp;
			  write_sp_flag = 1'b1;
			end  
    READ_SP:		begin
			   read_sp;
			   read_sp_flag = 1'b1;
			end   
    COPY_SP:		begin
                           copy_sp;
			   copy_sp_flag = 1'b1;
			end   
    READ_MEM:		read_mem;
    default:            report_invalid_opcode(OPCODE);
  endcase
  
  #1;
  write_sp_flag = 1'b0;
  read_sp_flag = 1'b0;
  copy_sp_flag = 1'b0;
  
end

///// Write scratchpad definition //////////////////////////////////////////////

task write_sp;

begin
  $strobe("%t OW_SLAVE - Write Scratchpad command received",$time);
  IOX_RDDATA = 1;				// Send ones from now till rst
  END_1WIRE = 1;				// 1-Wire is done
end
endtask


///// Read scratchpad definition ///////////////////////////////////////////////

task read_sp;

begin
  $strobe("%t OW_SLAVE - Read_Scratchpad command received",$time);
  IOX_RDDATA = 1;				// Send ones from now till rst 
  END_1WIRE = 1;				// 1-Wire is done 
end
endtask

///// Copy scratchpad definition ///////////////////////////////////////////////

task  copy_sp;

begin
    $strobe("%t OW_SLAVE - Copy_Scratchpad command received",$time);
    IOX_RDDATA = 1;
    IOX_READ = 1;
end
endtask


///// Read Memory definition ///////////////////////////////////////////////////

task read_mem;
begin
    $strobe("%t OW_SLAVE - Read_Memory command received",$time);
  IOX_RDDATA = 1;				// Send ones from now till rst  
  END_1WIRE = 1;				// 1-Wire is done
end
endtask


///// Report Invlid Opcode definition ///////////////////////////////////////////////////
task report_invalid_opcode(input [7:0] opcode);
begin

  $strobe("%t OW_SLAVE CMD_CTRL: INVALID OPCODE %h",$time,opcode);
    
  IOX_RDDATA = 1;				// Send ones from now till rst  
  END_1WIRE = 1;				// 1-Wire is done
end
endtask
endmodule
