module scoreboard (OWS_ROMID);

input [63:0] OWS_ROMID;


reg [7:0] scratchpad [0:1023];
reg [7:0] exp_cmd_fifo   [0:15];

reg [9:0] sp_wptr;
reg [9:0] sp_rptr;


integer sp_cnt;
integer cmd_cnt;

reg [3:0] cmd_fifo_wptr;
reg [3:0] cmd_fifo_rptr;

reg status;

initial begin
   sp_wptr = 0;
   sp_rptr = 0;
   sp_cnt  = 0;
   cmd_cnt = 0;
   cmd_fifo_wptr = 0;
   cmd_fifo_rptr = 0;
   status = 1;
end 
//-----------------------

task write_command_fifo (input [7:0] cmd);

begin

   exp_cmd_fifo[cmd_fifo_wptr] = cmd;
   cmd_cnt = cmd_cnt + 1;
   cmd_fifo_wptr = cmd_fifo_wptr + 1;

end

endtask

//-----------------------


task verify_command (input [7:0] cmd);

   reg [7:0] cmd;
begin
   if (cmd != exp_cmd_fifo[cmd_fifo_rptr]) begin
      $display("%t ERROR - Slave detected unexpected command:   expected command = %h, actual command = %h",$time, exp_cmd_fifo[cmd_fifo_rptr], cmd);
      status = 0;
   end
   cmd_cnt = cmd_cnt - 1;
   cmd_fifo_rptr = cmd_fifo_rptr + 1;
      
   
end

endtask

//-----------------------

task write_scratchpad (input [7:0] data);

begin

   scratchpad[sp_wptr] = data;
   sp_wptr = sp_wptr + 1;
   sp_cnt = sp_cnt + 1;

end

endtask

//-----------------------


task indexed_write_scratchpad (input [9:0] address, input [7:0] data);

begin
   sp_wptr = address;
   write_scratchpad(data);
end

endtask

//-----------------------

task verify_scratchpad (input [7:0] exp_data);

   reg [7:0] data;
begin

   data = scratchpad[sp_rptr];
   sp_rptr = sp_rptr + 1;
   sp_cnt = sp_cnt - 1;
   
   if (data != exp_data) begin
      $display("%t ERROR - Scratchpad data  address = %h, expected data = %h, actual data = %h",$time, sp_rptr, exp_data, data);
      status = 0;
   end

end

endtask

//-----------------------


task indexed_verify_scratchpad (input [9:0] address, input [7:0] exp_data);

begin
   sp_rptr = address;
   verify_scratchpad(exp_data);
end

endtask
//-----------------------


task verify_romid (input [63:0] romid);

begin
   if (romid != OWS_ROMID) begin
      $display("%t ERROR - Invalid ROMID  - expected  = %h, actual = %h",$time,  OWS_ROMID, romid);
      status = 0;
   end
end

endtask

//-----------------------

task report_status();

begin
  if (sp_cnt > 0) begin
     status = 0;
  end 
    
  if (cmd_cnt > 0) begin
     status = 0;
  end   
     
  $display("------------------------");
     
  if (status ==  1)
     $display("\t Test Passed");
  else    
     $display("\t Test Failed");
     
     
  $display("------------------------");
end
endtask
endmodule
